// Fragment Shader – file "base.frag"

#version 150 //SL version 2 not 4

uniform sampler2DShadow shadow_map;
uniform sampler2D tex0;
uniform sampler2D normalmap;
uniform sampler2D reflectionmap;



in vec3 fragEye;
in vec3 fragNormal;
in vec2 ex_fragTexCoord;
in vec3 light;
in vec4 sm_tex_coord;
in vec3 eyeVec;
in vec3 lv;
in vec4 old_Position;

uniform int mode;
uniform mat4 mvMat;
uniform mat4 M;
//uniform mat4 invmvpMat;
uniform float timer;


out vec4 out_Color;

void setDebugShading(){
out_Color = vec4(1.0,0.0,0.0,1.0); 
}

void main(void)
{
if (mode == 0)
	setDebugShading();
	else
	{
float inShadow = 1.0;
if(sm_tex_coord.w > 0.0){
	inShadow = textureProj(shadow_map, sm_tex_coord);
	}
	
	
	float diffuseIntensity;
	float specularIntensity;

	vec3 normal;
	vec3 eye;
	
	vec3 reflection;
	
	
	normal = normalize(fragNormal);
	eye = normalize(fragEye);
	diffuseIntensity = clamp(max(dot(normal, light), 0.0), 0.0, 1.0);
		
		
	if (mode == 5 || mode == 6){ // bump mapping
		vec2 disp;
		disp.x = 0.0f;
		disp.y = 0.0f;
		if(mode == 6){
			disp.x = sin(timer)/40.0f;
			disp.y = cos(timer)/40.0f;
		}
		normal = 2.0 * texture2D (normalmap, ex_fragTexCoord.st + disp).rgb - 1.0;
		normal = normalize((mvMat * vec4(normal,0)).xyz);
		eye = eyeVec;
		diffuseIntensity *= clamp(max(dot(normal,normalize(light)), 0.0), 0.0, 1.0);
	}



	
	
	if (mode == 2 || mode == 4 || mode == 6){ // specular
		reflection = normalize(reflect(-light, normal));
		specularIntensity = pow(clamp(max(dot(reflection, eye), 0.0), 0.0, 1.0), 50.0 );
	}else{
		specularIntensity = 0;
	}
	vec4 tex_color = texture2D(tex0, ex_fragTexCoord);
	vec4 final_color;
	
	if (mode == 0){
	final_color = vec4(tex_color.xyz * 0.6,tex_color.w);
	}else{
	final_color =  vec4(tex_color.xyz * diffuseIntensity * inShadow ,tex_color.w) + vec4(1.0, 0.9, 0.9, 1.0) * specularIntensity  + vec4(tex_color.xyz * 0.1,0.0);
	
	
	
	}
	// water fog stuff
	float z = (gl_FragCoord.z / gl_FragCoord.w);
	float fogFactor = exp2(-0.003 * 0.003 * z * z * 1.442695 );
	fogFactor = clamp(fogFactor, 0.2, 1.0);
	float y = (M * old_Position).y/3;
	if(y < -4.6){
	out_Color = mix(vec4(0.1, 0.1, 0.1, 1.0), final_color, fogFactor);
	}else out_Color = final_color;
	
	
	
	//out_Color = vec4(abs(y), abs(y), abs(y), 1.0);
	if(mode == 7)
	out_Color = final_color * vec4(1.0, 0.0, 0.0, 1.0);

}
}
