#version 150 

uniform sampler2D tex0;
uniform sampler2D tex1;
uniform sampler2D tex2;
uniform sampler2D tex3;
uniform samplerCube envMap;

in vec3 fragEye;
in vec3 fragNormal;
in vec2 ex_fragTexCoord;
in vec3 light[5];
in float dist[5];
in float alpha;

uniform int mode;
uniform mat4 mvMat;
uniform bool selected;
uniform int nrLights;

uniform float energy[5];



out vec4 out_Color;



void main(void)
{

	
	float diffuseIntensity;
	float specularIntensity;
	float ambientIntensity;
	
	vec4 ambientColor = vec4(0);
	vec4 specularColor = vec4(0);
	vec4 envMapColor = vec4(0);
	vec4 finalColor = vec4(0);
	vec4 selectionColor = vec4(0);
		
	vec3 normal;
	vec3 eye;
	vec3 reflection;
	
	//attenuation factors
	float att;
	float Kc = 0.01f;
	float Kl = 0.0f;
	float Kq = 0.0002f;
	
	ambientIntensity = 0.1;
	ambientColor = vec4(1,1,1,1);
	
	
	out_Color = vec4(0);
	
	for(int i=0; i<nrLights; i++)
	{
		normal = normalize(fragNormal);
		eye = normalize(fragEye);
		diffuseIntensity = clamp(max(dot(normal, light[i]), 0.0), 0.0, 1.0);
			
		att = clamp(1.0/(Kc + Kl * dist[i] + Kq * dist[i] * dist[i]),0.0,1.0);
		
		reflection = normalize(reflect(-light[i], normal));
		specularIntensity = pow(clamp(max(dot(reflection, eye), 0.0), 0.0, 1.0), 50.0 );
		specularColor = vec4(1.0, 0.9, 0.9, 1.0);

		vec4 tex_color = texture(tex0, ex_fragTexCoord);
		
		
		if(mode == 6)
		{
			selectionColor = vec4(0.2, 0.2, 0.2 ,0.0); 	//object selected

		}
		if(mode == 7)
		{
			selectionColor = vec4(0.0, 0.5, 0.1 ,0.0); 	//object selected
		}
		if(mode == 8)
		{
			selectionColor = vec4(1.0, 0.0, 0.0 ,0.0); 	//object selected
		}
		if(mode == 4)
		{
			envMapColor = texture(envMap, reflect(-eye, normal));
		}
		float eg = 1.0f;
		if(i>0)eg = energy[i];
		float texw = max(tex_color.w, tex_color.w * att);
		finalColor = (ambientIntensity * tex_color) + vec4(tex_color.xyz * (diffuseIntensity + specularIntensity) * att * eg, texw ) + selectionColor;
		envMapColor = (ambientIntensity * envMapColor) + envMapColor * (diffuseIntensity + specularIntensity) * att * eg  + selectionColor;
		envMapColor.w = 1.0;
		
		if(mode == 4) 
		{
		finalColor = mix(finalColor,envMapColor, 0.5);
		finalColor.w *= alpha;
		out_Color += finalColor;
		
		}else out_Color += finalColor;


	}
	out_Color = out_Color;

}
